(*
Notice of Copyright Ownership

Copyright (c) 2003-2010 Jensen Data Systems. Inc.

This code was produced by and is copyrighted by Jensen Data Systems, Inc. All data 
mentioned or referred to, including names, addresses, phone numbers, descriptions, 
and images, are fictitious. Any resemblance to actual persons, companies, and products 
is purely coincidental.

This code is intended to be used in conjunction with database examples and 
descriptions found in the book: 
1. Advantage Database Server: A Developer's Guide, 2nd Edition 
by Cary Jensen, Ph.D. and Loy Anderson, Ph.D. (2010, Jensen Data Systems, Inc.)

For information on purchasing this book, visit:
http://www.JensenDataSystems.com/ADSBook10
*)

library AEPDemoDelphi;

{$INCLUDE versions.inc}

{$IFDEF ADSDELPHI7_OR_NEWER}
   {$WARN UNSAFE_TYPE OFF}
   {$WARN UNSAFE_CODE OFF}
   {$WARN UNSAFE_CAST OFF}
{$ENDIF}

uses
  SysUtils,
  Windows,
  Classes,
  ace,
  adsdata,
  adstable,
  dm in 'dm.pas' {dm1: TDataModule},
  AdsAEPSessionMgr;

{$E aep}

{$IFDEF LINUX}
{$SOPREFIX ''}
{$ENDIF}

var
   AEPSessionMgr : TAdsAEPSessionMgr;


{*******************************************************************************
 * This is the AEP startup function.  This is called the first time an Advantage
 * connection calls a procedure in this AEP module. It is called once for each
 * Advantage connection that calls a procedure in this module.
 * Do any per-connection initialization here. Use the ulConnectionID as a
 * unique identifier for each connection.
 ******************************************************************************}
function Startup( ulConnectionID: UNSIGNED32;
                  hConnection: ADSHANDLE
                ): UNSIGNED32;
                {$IFDEF WIN32}stdcall;{$ENDIF}{$IFDEF LINUX}cdecl;{$ENDIF} // Do not change the prototype.
var
   DM1 : TDM1;
begin

   DM1 := nil;
   Result := AE_SUCCESS;

   try
      DM1 := TDM1.Create( nil );

      {* Configure DataConn to use the active handle hConnection. *}
      DM1.DataConn.SetHandle( hConnection );
      {* No need to activate the connection, SetHandle does that automatically. *}

      {* Place tables on the data module, assign DataConn as their Databasname
       * property, and then open them here. *}

      {* Add this data module to the session manager, so we can retrieve it for
       * the user identified by ulConnectionID the next time they need it. *}
      AEPSessionMgr.AddDM( ulConnectionID, DM1 );

   except
      on E : EAdsDatabaseError do
         {* ADS-specific error, use ACE error code *}
         if assigned( DM1 ) then
            DM1.DataConn.Execute( 'INSERT INTO __error VALUES ( ' + IntToStr( E.ACEErrorCode ) + ', ' + QuotedStr( E.Message ) + ' )' )
         else
            Result := E.ACEErrorCode;
      on E : exception do
         {* other error *}
         if assigned( DM1 ) then
            DM1.DataConn.Execute( 'INSERT INTO __error VALUES ( 1, ' + QuotedStr( E.Message ) + ' )' )
         else
            Result := 1;
   end;

end;



{*******************************************************************************
 * This is the AEP shutdown function.  This is called once for each Advantage
 * connection that has called a procedure in this module, and is called when
 * the connection is terminating.
 * The prototype must be exactly as it is in the example.
 * Do your per-connection clean-up here. Use the ulConnectionID as a
 * unique identifier for each connection.
 ******************************************************************************}
function Shutdown( ulConnectionID: UNSIGNED32;
                   hConnection: ADSHANDLE
                 ): UNSIGNED32;
                 {$IFDEF WIN32}stdcall;{$ENDIF}{$IFDEF LINUX}cdecl;{$ENDIF} // Do not change the prototype.
begin

   try
      Result := AE_SUCCESS;

      if assigned( AEPSessionMgr ) then
         AEPSessionMgr.FreeDM( ulConnectionID );

   except
      on E : EAdsDatabaseError do
         Result := E.ACEErrorCode;
      on E : exception do
         result := 1;
   end;

end;



{* Sample exported Advantage Extended Procedure. If you change the name of this
 * procedure remember to also change the name in the exports list at the bottom
 * of this file. *}
function Get10Percent( ulConnectionID: UNSIGNED32;
                      hConnection: ADSHANDLE;
                      pulNumRowsAffected: PUNSIGNED32 ):
                      UNSIGNED32;
                      {$IFDEF WIN32}stdcall;{$ENDIF}
                      {$IFDEF LINUX}cdecl;{$ENDIF}
var
   DM1 : TDM1;
   AdsQuery1: TAdsQuery;
   CustID: Integer;
   RecCount: Integer;
begin

   Result := AE_SUCCESS;

   {* Get this connection's data module from
    the session manager. *}
   DM1 := TDM1( AEPSessionMgr.GetDM( ulConnectionID ) );
   try
     AdsQuery1 := TAdsQuery.Create(nil);
     try
       //Get customer's invoices
       DM1.tblInput.open;
       CustID := DM1.tblInput.Fields[0].AsInteger;
       DM1.tblInput.Close;
       AdsQuery1.AdsConnection := DM1.DataConn;
       AdsQuery1.SQL.Text := 'SELECT Count(*) FROM INVOICE ' +
                             '  WHERE [Customer ID] = :custid';
       AdsQuery1.Params[0].AsInteger := CustID;
       AdsQuery1.Open;
       RecCount := AdsQuery1.Fields[0].AsInteger;
       AdsQuery1.Close;

       if RecCount < 10 then
       begin
         DM1.DataConn.Execute( 'INSERT INTO __error '+
           'VALUES ( 2501, ' +
           QuotedStr( 'There are less than 10 ' +
           'records for ' +
           IntToStr(CustID) ) + ' )' );
         Exit;
       end;

       AdsQuery1.SQL.Text := 'INSERT INTO __output ' +
                             'SELECT TOP 10 Percent [Invoice No] ' +
                             '  FROM INVOICE ' +
                             '  WHERE [Customer ID] = ' + IntToStr(CustID) +
                             '  ORDER BY Rand();' ;
       AdsQuery1.ExecSQL;
     finally
       AdsQuery1.Free;
     end;

   except
      on E : EADSDatabaseError do
         {* ADS-specific error, use ACE error code *}
         DM1.DataConn.Execute( 'INSERT INTO __error '+
           'VALUES ( ' + IntToStr( E.ACEErrorCode ) +
           ', ' + QuotedStr( E.Message ) + ' )' );
      on E : Exception do
         {* other error *}
         DM1.DataConn.Execute( 'INSERT INTO __error '+
          'VALUES ( 2500, ' + QuotedStr( E.Message ) + ' )' );
   end;
end; //Get10Percent


{* Do not modify this function, it is used internally by the Advantage server. *}
function GetInterfaceVersion : UNSIGNED32;
{$IFDEF WIN32}stdcall;{$ENDIF}{$IFDEF LINUX}cdecl;{$ENDIF} // Do not change the prototype.
begin
   Result := 3;
end;


{* Put any global clean-up code in the DLL_PROCESS_DETACH code. *}
procedure DllMain(reason: integer) ;
begin
   case reason of
     DLL_PROCESS_DETACH:
     begin
        {* Place global clean-up code in this case. *}
        AEPSessionMgr.Free;
     end;
   end;
end; (*DllMain*)

exports
   GetInterfaceVersion,
   Get10Percent,
   Startup,
   Shutdown;


{* Put any global initialization code here. *}
begin
   {* Let the VCL memory manager know we will be used by a multi-threaded exe. *}
   IsMultiThread := TRUE;

   {* Initialize the session manager for this dll here, where it's thread-safe, as
    * opposed to inside of the Startup function, which can be called by multiple
    * Advantage threads at the same time. *}
   AEPSessionMgr := TAdsAEPSessionMgr.Create( nil );

   {* Assign our own cleanup procedure. *}
   DllProc := @DllMain;
end.
