(*
Notice of Copyright Ownership

Copyright (c) 2003-2010 Jensen Data Systems. Inc.

This code was produced by and is copyrighted by Jensen Data Systems, Inc. All data 
mentioned or referred to, including names, addresses, phone numbers, descriptions, 
and images, are fictitious. Any resemblance to actual persons, companies, and products 
is purely coincidental.

This code is intended to be used in conjunction with database examples and 
descriptions found in the book: 
1. Advantage Database Server: A Developer's Guide, 2nd Edition 
by Cary Jensen, Ph.D. and Loy Anderson, Ph.D. (2010, Jensen Data Systems, Inc.)

For information on purchasing this book, visit:
http://www.JensenDataSystems.com/ADSBook10
*)

// Copyright (c) 2003-2007 Extended Systems, Inc.
// Portions Copyright (c) 2005-2010, iAnywhere Solutions, Inc.
// All rights reserved. All unpublished rights reserved.
//
// This source code can be used, modified, or copied by the licensee as long as
// the modifications (or the new binary resulting from a copy or modification of
// this source code) are used with Extended Systems' products.
//
// Extended Systems Inc. does not warrant that the operation of this software
// will meet your requirements or that the operation of the software will be
// uninterrupted, be error free, or that defects in software will be corrected.
// This software is provided "AS IS" without warranty of any kind. The entire
// risk as to the quality and performance of this software is with the purchaser.
// If this software proves defective or inadequate, purchaser assumes the entire
// cost of servicing or repair. No oral or written information or advice given
// by an Extended Systems Inc. representative shall create a warranty or in any
// way increase the scope of this warranty.

library TriggerDemoDelphi;

{$INCLUDE versions.inc}

{$IFDEF ADSDELPHI7_OR_NEWER}
   {$WARN UNSAFE_TYPE OFF}
   {$WARN UNSAFE_CODE OFF}
   {$WARN UNSAFE_CAST OFF}
{$ENDIF}

uses
  SysUtils,
  Classes,
  ace,
  adscnnct,
  adsset,
  adsdata,
  adstable;

// Utility Function Prototype
procedure SetError ( conn : TAdsConnection; code : UNSIGNED32; err  : string ); forward;



// Sample Advantage Trigger function. If you change the name of this
// function, remember to also change the name in the exports list at the bottom
// of this file.
function GenGuidID
(
  ulConnectionID : UNSIGNED32; // (I) Unique ID identifying the user causing this trig
  hConnection    : ADSHANDLE;  // (I) Active ACE connection handle user can perform
                               //     operations on
  pcTriggerName  : PAceChar;   // (I) Name of the trigger object in the dictionary
  pcTableName    : PAceChar;   // (I) Name of the base table that caused the trigger
  ulEventType    : UNSIGNED32; // (I) Flag with event type (insert, update, etc.)
  ulTriggerType  : UNSIGNED32; // (I) Flag with trigger type (before, after, etc.)
  ulRecNo        : UNSIGNED32  // (I) Record number of the record being modified
) : UNSIGNED32;
{$IFDEF WIN32}stdcall;{$ENDIF}
{$IFDEF LINUX}cdecl;{$ENDIF}
var
  //This variable was in the original template
  oConn  : TAdsConnection;

  //These variables were added
  oTable: TAdsTable;
  oNewTable: TAdsTable;
  GUID1 : TGUID;
  GUIDStr: String;
  i: Integer;

begin
  Result := 0;

  //This code was part of the original template
  oConn := TAdsConnection.CreateWithHandle( nil,
    hConnection);

  //This table will point to the table
  //to which the trigger is assigned
  oTable := TAdsTable.Create(nil);

  //This table will point to the
  //single record that is being inserted data
  oNewTable := TAdsTable.Create(nil);

  try
    try

      //Hook up both AdsTables to the connection
      //that was passed to the trigger
      oTable.AdsConnection := oConn;
      oNewTable.AdsConnection := oConn;

      //Hook the AdsTables to the
      //appropriate underlying table
      oNewTable.TableName := '__new';
      oTable.TableName := AnsiString(pcTableName);

      //Open the tables
      oTable.Open;
      oNewTable.Open;

      //Add a new record to the table to which
      //the trigger is assigned. This new
      //record will be the 'inserted' record.
      oTable.Append;

      //Generate a GUID as a string
      if CreateGUID(GUID1) = 0 then
        GUIDStr := GUIDToString(GUID1);

      //If the first field has not already been assigned
      //a value, insert the GUID string
      if (oNewTable.Fields[0].AsString = '') or
         (oNewTable.Fields[0].IsNull) then
        oTable.Fields[0].AsString := GUIDStr
      else
        oTable.Fields[0].Value := oNewTable.Fields[0].Value;

      //Now iterate through the remaining fields
      //and insert the data. Only do this
      //for non-null fields
      for i := 1 to Pred(oTable.FieldCount) do
        if not oNewTable.Fields[i].IsNull then
          oTable.Fields[i].Value :=
            oNewTable.Fields[i].Value;

      //Post the data.
      oTable.Post;

      //All done. Goto the finally clause and clean-up.
    except
      on E : EADSDatabaseError do
        SetError( oConn, E.ACEErrorCode, E.message );
      on E : Exception do
        SetError( oConn, 0, E.message );
    end;

  finally
    //Close and free the tables.
    //Also free the connection object
    oTable.Close;
    oNewTable.Close;
    oConn.Free;
    oTable.Free;
    oNewTable.Free;
  end;
end;


// Utility function to return an error from a trigger.
procedure SetError
(
  conn : TAdsConnection;
  code : UNSIGNED32;
  err  : string
);
begin
  // Errors can be returned by placing a row into the __error table.
  conn.Execute( 'INSERT INTO __error VALUES( ' + IntToStr( code ) +
                ', ' + QuotedStr( err ) + ' )' );
end;


{* Do not modify this function, it is used internally by the Advantage server. *}
function GetInterfaceVersion : UNSIGNED32;
{$IFDEF WIN32}stdcall;{$ENDIF}{$IFDEF LINUX}cdecl;{$ENDIF} // Do not change the prototype.
begin
   Result := 3;
end;


exports
  GenGuidID;

begin
  // Because this dll is used by a multi-threaded application (the Advantage
  // server), we must set the Delphi IsMultiThread global variable to TRUE.
  IsMultiThread := TRUE;
end.
